import { TextProps, StyleProp, ViewStyle, ViewProps, TouchableOpacityProps } from 'react-native';
import * as react from 'react';

type IBoldProps = {
    fontFamily?: string;
} & TextProps;

declare const Bold: ({ fontFamily, children }: IBoldProps) => JSX.Element;

type ICodeProps = TextProps;

declare const Code: ({ children }: ICodeProps) => JSX.Element;

/** Reference https://github.com/editor-js/delimiter */
type IDelimiterProps = {
    data: Record<string, never>;
    containerStyle?: StyleProp<ViewStyle>;
} & Omit<ViewProps, 'style'>;

/** Reference https://github.com/editor-js/delimiter */
declare const Delimiter: ({ containerStyle, ...rest }: IDelimiterProps) => JSX.Element;

type IFallbackBlockProps = {
    blockType: string;
    containerStyle?: StyleProp<ViewStyle>;
} & Omit<ViewProps, 'style'>;

declare const FallbackBlock: ({ blockType, containerStyle, ...rest }: IFallbackBlockProps) => JSX.Element;

type IHeaderProps = {
    data: {
        level: 1 | 2 | 3 | 4 | 5 | 6;
        text: string;
    };
    fontFamily?: string;
} & TextProps;

declare const Header: ({ data, fontFamily, style, ...rest }: IHeaderProps) => JSX.Element;

type IImageFrameProps = {
    data: {
        file: {
            url: string;
        };
        caption?: string;
        withBorder?: boolean;
        withBackground?: boolean;
        stretched?: boolean;
    };
    captionFontFamily?: string;
    containerStyle?: StyleProp<ViewStyle>;
} & Omit<ViewProps, 'style'>;

declare const ImageFrame: ({ data, captionFontFamily, containerStyle, ...rest }: IImageFrameProps) => JSX.Element;

type IItalicProps = {
    fontFamily?: string;
} & TextProps;

declare const Italic: ({ children, fontFamily }: IItalicProps) => JSX.Element;

type ILinkToolProps = {
    data: {
        link: string;
        meta: {
            title?: string;
            site_name?: string;
            description?: string;
            image?: {
                url?: string;
            };
        };
    };
    containerStyle?: StyleProp<ViewStyle>;
} & Omit<TouchableOpacityProps, 'style'>;

declare const LinkTool: ({ data, containerStyle, ...rest }: ILinkToolProps) => JSX.Element;

type IListProps = {
    data: {
        items: Array<string>;
        style: 'ordered' | 'unordered';
    };
    fontFamily?: string;
    containerStyle?: StyleProp<ViewStyle>;
} & Omit<ViewProps, 'style'>;

declare const List: ({ data, fontFamily, containerStyle }: IListProps) => JSX.Element;

type IMarkProps = TextProps;

declare const Mark: ({ children }: IMarkProps) => JSX.Element;

type IParagraphProps = {
    data: {
        text: string;
    };
    fontFamily?: string;
} & TextProps;

declare const Paragraph: ({ data, fontFamily, style, ...rest }: IParagraphProps) => JSX.Element;

type IPersonalityProps = {
    data: {
        name?: string;
        description?: string;
        link: string;
        photo?: string;
    };
    containerStyle?: StyleProp<ViewStyle>;
} & Omit<TouchableOpacityProps, 'style'>;

declare const Personality: ({ data, containerStyle, ...rest }: IPersonalityProps) => JSX.Element;

type IQuoteProps = {
    data: {
        text: string;
        caption?: string;
        alignment?: 'left' | 'center';
    };
    quoteFontFamily?: string;
    captionFontFamily?: string;
    containerStyle?: StyleProp<ViewStyle>;
} & Omit<ViewProps, 'style'>;

declare const Quote: ({ data, quoteFontFamily, captionFontFamily, containerStyle, ...rest }: IQuoteProps) => JSX.Element;

type ISimpleImageProps = {
    data: {
        url: string;
        caption?: string;
        withBorder?: boolean;
        withBackground?: boolean;
        stretched?: boolean;
    };
    captionFontFamily?: string;
    containerStyle?: StyleProp<ViewStyle>;
} & Omit<TextProps, 'style'>;

declare const SimpleImage: ({ data, captionFontFamily, containerStyle, ...rest }: ISimpleImageProps) => JSX.Element;

type IUnderlineProps = {
    fontFamily?: string;
} & TextProps;

declare const Underline: ({ fontFamily, children }: IUnderlineProps) => JSX.Element;

interface IEditorJsData {
    /**
     * Editor's version
     */
    version?: string;
    /**
     * Timestamp of saving in milliseconds
     */
    time?: number;
    /**
     * Saved Blocks
     */
    blocks: OutputBlockData[];
}
interface OutputBlockData<Type extends string = string, Data extends object = any> {
    /**
     * Unique Id of the block
     */
    id?: string;
    /**
     * Tool type
     */
    type: Type;
    /**
     * Saved Block data
     */
    data: BlockToolData<Data>;
    /**
     * Block Tunes data
     */
    tunes?: {
        [name: string]: BlockTuneData;
    };
}
type BlockToolData<T extends object = any> = T;
type BlockTuneData = any;

interface IComponentBlockProps<Data extends object = any> {
    block: OutputBlockData<string, Data>;
    containerStyle: StyleProp<ViewStyle>;
}
interface IComponentObject {
    [key: string]: (param: IComponentBlockProps) => JSX.Element;
}
interface ICreateEditorJsViewerProps {
    tools?: IToolsParser;
    customTools?: ICustomToolsParser;
    /** Show a fallback Component when type of block is `unknown`. Default is `false`*/
    showBlockFallback?: boolean;
}
interface IToolsParser {
    delimiter?: {
        /**
         * A component with type {@link IDelimiterProps} or any for your custom header tool
        */
        Component?: (props: IDelimiterProps | any) => JSX.Element;
    };
    header?: {
        /**
         * A component with type {@link IHeaderProps} or any for your custom header tool
        */
        Component?: (props: IHeaderProps | any) => JSX.Element;
        /**
         * This prop will be ignored if you use a CustomComponent
        */
        fontFamily?: string;
    };
    image?: {
        /**
         * A component with type {@link IImageFrameProps} or any for your custom image tool
        */
        Component: (props: IImageFrameProps | any) => JSX.Element;
        /**
         * This prop will be ignored if you use a CustomComponent
        */
        captionFontFamily?: string;
    };
    linkTool?: {
        /**
         * A component with type {@link ILinkToolProps} or any for your custom linkTool tool
        */
        Component?: (props: ILinkToolProps | any) => JSX.Element;
    };
    /**
     * Is not recommended to replace this component because he need a html parser to RN JSX
    */
    list?: {
        /**
         * A component with type {@link IListProps} or any for your custom list tool
        */
        Component?: (prop: IListProps | any) => JSX.Element;
        /**
         * This prop will be ignored if you use a CustomComponent
        */
        fontFamily?: string;
    };
    /**
     * Is not recommended to replace this component because he need a html parser to RN JSX
    */
    paragraph?: {
        /**
         * A component with type {@link IParagraphProps} or any for your custom paragraph tool
        */
        Component?: (props: IParagraphProps | any) => JSX.Element;
        /**
        * This prop will be ignored if you use a CustomComponent
       */
        fontFamily?: string;
    };
    personality?: {
        /**
         * A component with type {@link IPersonalityProps} or any for your custom paragraph tool
        */
        Component?: (props: IPersonalityProps | any) => JSX.Element;
    };
    simpleImage?: {
        /**
         * A component with type {@link ISimpleImageProps } or any for your custom simpleImage tool
        */
        Component?: (props: ISimpleImageProps | any) => JSX.Element;
        /**
         * This prop will be ignored if you use a CustomComponent
        */
        captionFontFamily?: string;
    };
    quote?: {
        /**
         * A component with type {@link IQuoteProps } or any for your custom simpleImage tool
        */
        Component?: (props: IQuoteProps | any) => JSX.Element;
        /**
         * This prop will be ignored if you use a CustomComponent
        */
        quoteFontFamily?: string;
        /**
         * This prop will be ignored if you use a CustomComponent
        */
        captionFontFamily?: string;
    };
}
interface ICustomToolsParser {
    [key: string]: {
        Component?: (param: IComponentBlockProps) => JSX.Element;
    };
}

type IEditorJsViwerNativeProps = {
    data: IEditorJsData;
} & Pick<ViewProps, 'style'>;

declare const createEditorJsViewer: (props?: ICreateEditorJsViewerProps) => react.MemoExoticComponent<({ data, ...rest }: IEditorJsViwerNativeProps) => JSX.Element | null>;

export { Bold, Code, Delimiter, FallbackBlock, Header, IBoldProps, ICodeProps, IComponentBlockProps, IComponentObject, ICreateEditorJsViewerProps, ICustomToolsParser, IDelimiterProps, IEditorJsData, IEditorJsViwerNativeProps, IFallbackBlockProps, IHeaderProps, IImageFrameProps, IItalicProps, ILinkToolProps, IListProps, IMarkProps, IParagraphProps, IPersonalityProps, IQuoteProps, ISimpleImageProps, IToolsParser, IUnderlineProps, ImageFrame, Italic, LinkTool, List, Mark, Paragraph, Personality, Quote, SimpleImage, Underline, createEditorJsViewer };
